#
# Ubuntu PractiTest Community results processor
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import datetime
import json
import logging
import os
import pymongo
from flask import request

from qakit.practitest import practitest
from qakit.practitest.report_results_to_practitest import _read_config
from qakit.practitest.report_checkbox_results_to_practitest import (
    upload_results
)
from ubuntu_pt_community import auth, db

logger = logging.getLogger(__name__)

api_preamble = 'v1'


def define_routes(webapp):
    webapp.add_url_rule(
        '/{}/upload'.format(api_preamble),
        view_func=upload,
        methods=['POST']
    )


def upload():
    # Steps:
    #  - Authenticate request (i.e. do we know this person and care?)
    #  - Grab uploaded data
    #  - Upload to PT
    #  ? Keep local (raw) copy?

    # First pass lets explicitly do the auth stuff, then we can do something
    # smarter and more elegant.
    # Currently will blow up if if doesn't validate.
    # auth.validate_request(request)
    pt = practitest.PractitestSession(
        **_read_config(os.path.join(
            os.environ['CONFIG_DIR'], 'config.ini'
        ))
    )
    try:
        upload_json_data = request.files['data'].stream.getvalue()
        upload_results(json.loads(upload_json_data.decode('utf-8')), pt)
        store_upload_details(request, upload_json_data)
    except KeyError:
        print('No files provided.')
    return 'Uploading successful.'


def store_upload_details(request, json_data):
    user_details = get_user_details_from_header(request)

    try:
        email_address = user_details['email']
    except KeyError:
        logger.error(
            'Attempted to store details for results with no user details'
        )
        return

    details = dict(
        user_email=email_address,
        uploaded=datetime.datetime.utcnow(),
        results=json_data
    )

    try:
        collection = get_results_database()
        insert_id = collection.insert(details)
        logger.info(
            'Inserted details for {} with id {}'.format(
                email_address,
                insert_id
            )
        )
    except pymongo.errors.DuplicateKeyError:
        # This shouldn't happen due to the timestamp.
        logger.error('Attempting to store duplicate results')
    except Exception as e:
        logger.error('Failed to get database connection: ', e)


def get_results_database():
    database_name = 'community_practitest'
    collection_name = 'uploaded_results'
    return db.get_collection(database_name, collection_name)


def get_user_details_from_header(request):
    # STUB
    return dict(email='another@email.com')
