#
# Ubuntu PractiTest Community results processor
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import configparser
import logging
import os
import pymongo


logger = logging.getLogger(__name__)


def get_database_client():
    """Return a MongoClient for the mongodb for this service."""
    config = get_config()
    return pymongo.MongoClient(config['address'], config['port'])


def get_database(database_name):
    """Return a pymongo.Database of name `database_name`"""
    client = get_database_client()
    return client[database_name]


def get_collection(database_name, collection_name):
    """Return a pymongo.Collection named `collection_name` found in
    `database_name`.

    """
    db = get_database(database_name)
    return db[collection_name]


def get_config_file_path():
    try:
        return os.path.join(
            os.environ['SERVICE_DIR'],
            'ubuntu-pt-community.conf'
        )
    except KeyError:
        logger.error('Failed to find the service directory.')
        raise RuntimeError('Failed to find the service directory.')


def get_config():
    # Open file and read in the details.
    try:
        config_file_path = get_config_file_path()
        parser = configparser.ConfigParser()
        config = parser.read(config_file_path)
        return dict(config['mongodb'])
    except RuntimeError:
        # For local testing, should handle this better.
        return dict(address='localhost', port=27017)
